using Serilog;
using System.Globalization;
using System.IO.Compression;
using System.Text.RegularExpressions;

namespace File_Folder_Helper.Helpers;

internal static class HelperZipFilesByDate
{

    internal static void ZipFilesByDate(ILogger log, string sourceDirectory, SearchOption searchOption = SearchOption.TopDirectoryOnly, string dayFormat = "")
    {
        string key;
        bool addFile;
        string fileName;
        string[] segments;
        string[] subFiles;
        string weekOfYear;
        FileInfo fileInfo;
        DateTime creationTime;
        DateTime lastWriteTime;
        DateTime nowDateTime = DateTime.Now;
        Regex regex = new("[a-zA-Z0-9]{1,}");
        DateTime dateTime = DateTime.MinValue;
        DateTime firstEmail = new(2019, 3, 8);
        CultureInfo cultureInfo = new("en-US");
        Calendar calendar = cultureInfo.Calendar;
        Dictionary<string, DateTime> weeks = new();
        int ticksLength = nowDateTime.AddDays(-6).Ticks.ToString().Length;
        for (int i = 0; i < int.MaxValue; i++)
        {
            dateTime = firstEmail.AddDays(i);
            if (dateTime > nowDateTime)
                break;
            weekOfYear = calendar.GetWeekOfYear(dateTime, CalendarWeekRule.FirstDay, DayOfWeek.Sunday).ToString("00");
            key = string.Concat(dateTime.ToString("yyyy"), "_Week_", weekOfYear);
            if (!weeks.ContainsKey(key))
                weeks.Add(key, dateTime);
        }
        weekOfYear = calendar.GetWeekOfYear(nowDateTime, CalendarWeekRule.FirstDay, DayOfWeek.Sunday).ToString("00");
        string skipKey = string.Concat(nowDateTime.ToString("yyyy"), "_Week_", weekOfYear);
        Dictionary<string, List<string>> keyValuePairs = new();
        string[] topDirectories = Directory.GetDirectories(sourceDirectory, "*", SearchOption.TopDirectoryOnly);
        if (topDirectories.Length == 0)
            topDirectories = new string[] { sourceDirectory };
        foreach (string topDirectory in topDirectories)
        {
            keyValuePairs.Clear();
            subFiles = Directory.GetFiles(topDirectory, "*", searchOption);
            foreach (string subFile in subFiles)
            {
                addFile = false;
                if (subFile.EndsWith(".zip"))
                    continue;
                fileName = Path.GetFileName(subFile);
                fileInfo = new FileInfo(subFile);
                creationTime = fileInfo.CreationTime;
                if (creationTime > dateTime)
                    continue;
                lastWriteTime = fileInfo.LastWriteTime;
                if (fileName.Contains(lastWriteTime.ToString("yyyyMMdd")) || fileName.Contains(lastWriteTime.ToString("yyyy-MM-dd")) ||
                    fileName.Contains(creationTime.ToString("yyyyMMdd")) || fileName.Contains(creationTime.ToString("yyyy-MM-dd")) ||
                    fileName.Contains(lastWriteTime.ToString("yyMMdd")) || fileName.Contains(lastWriteTime.ToString("yy-MM-dd")) ||
                    fileName.Contains(creationTime.ToString("yyMMdd")) || fileName.Contains(creationTime.ToString("yy-MM-dd")) ||
                    fileName.Contains(lastWriteTime.AddDays(-1).ToString("yyyyMMdd")) || fileName.Contains(lastWriteTime.AddDays(-1).ToString("yyyy-MM-dd")) ||
                    fileName.Contains(creationTime.AddDays(-1).ToString("yyyyMMdd")) || fileName.Contains(creationTime.AddDays(-1).ToString("yyyy-MM-dd")) ||
                    fileName.Contains(lastWriteTime.AddDays(-1).ToString("yyMMdd")) || fileName.Contains(lastWriteTime.AddDays(-1).ToString("yy-MM-dd")) ||
                    fileName.Contains(creationTime.AddDays(-1).ToString("yyMMdd")) || fileName.Contains(creationTime.AddDays(-1).ToString("yy-MM-dd")))
                    addFile = true;
                if (!addFile && fileName.Length > ticksLength)
                {
                    MatchCollection matches = regex.Matches(fileName);
                    foreach (Match match in matches.Cast<Match>())
                    {
                        if (match.Value.Length != ticksLength)
                            continue;
                        if (!long.TryParse(match.Value, out long ticks))
                            continue;
                        addFile = true;
                        break;
                    }
                    if (addFile)
                        break;
                }
                if (addFile)
                {
                    weekOfYear = calendar.GetWeekOfYear(lastWriteTime, CalendarWeekRule.FirstDay, DayOfWeek.Sunday).ToString("00");
                    if (string.IsNullOrEmpty(dayFormat))
                        key = string.Concat(lastWriteTime.ToString("yyyy"), "_Week_", weekOfYear);
                    else
                        key = string.Concat(lastWriteTime.ToString("yyyy"), "_Week_", weekOfYear, "_", lastWriteTime.ToString(dayFormat));
                    if (key == skipKey)
                        continue;
                    if (!keyValuePairs.ContainsKey(key))
                        keyValuePairs.Add(key, new List<string>());
                    keyValuePairs[key].Add(subFile);
                }
            }
            foreach (KeyValuePair<string, List<string>> element in keyValuePairs)
            {
                key = Path.Combine(topDirectory, $"{element.Key}.zip");
                if (File.Exists(key))
                {
                    for (short i = 101; i < short.MaxValue; i++)
                    {
                        key = Path.Combine(topDirectory, $"{element.Key}_{i}.zip");
                        if (!File.Exists(key))
                            break;
                    }
                }
                using ZipArchive zip = ZipFile.Open(key, ZipArchiveMode.Create);
                foreach (string file in element.Value)
                {
                    _ = zip.CreateEntryFromFile(file, Path.GetFileName(file));
                    File.Delete(file);
                }
            }
            subFiles = Directory.GetFiles(topDirectory, "*.zip", SearchOption.TopDirectoryOnly);
            foreach (string subFile in subFiles)
            {
                fileName = Path.GetFileNameWithoutExtension(subFile);
                segments = fileName.Split('_');
                if (segments.Length > 2)
                    fileName = string.Concat(segments[0], '_', segments[1], '_', segments[2]);
                if (weeks.ContainsKey(fileName))
                {
                    try
                    { File.SetLastWriteTime(subFile, weeks[fileName]); }
                    catch (Exception) { }
                }
            }
            if (topDirectory != sourceDirectory)
            {
                try
                { _ = HelperDeleteEmptyDirectories.DeleteEmptyDirectories(topDirectory); }
                catch (Exception) { }
            }
            log.Information(topDirectory);
        }
    }

}